options(java.parameters = "-Xmx6g")

library(raster)
library(rgdal)
library(maptools)
library(rgeos)
library(sp)
library(ggplot2)
library(ggrepel)
library(BiodiversityR)
library(dismo)
library(compiler)

#DATA INPUT####
#set working directory
setwd("C:/Users/CR-C003/Dropbox/Gap analysis/Vigna")

#Enter Vigna records
vigna <- read.csv("Vigna_clean_5.csv",sep=";")

#Ensure that coordinates are numeric variables and remove NAs
vigna$decimalLongitude <- as.numeric(as.character(vigna$decimalLongitude))
vigna <- vigna[!is.na(vigna$decimalLongitude),]

vigna$decimalLatitude <- as.numeric(as.character(vigna$decimalLatitude))
vigna <- vigna[!is.na(vigna$decimalLongitude),]

#species not included in the analysis
spp_na <- c("V. candida", "V. gentryi", "v. juncea", "V. triphylla", 
            "V. kokii", "V. lanceolata", "V. nigritia", 
            "V. schimperi","V. spp.", "V. venusta")

#Remove selected species
for (i in 1:length(spp_na)){vigna <- subset(vigna,!GRIN.taxa == as.character(spp_na[i]))}

#Enter country data
setwd("C:/Users/CR-C003/Dropbox/World")
ctries <- readOGR("World_Countries.shp", "World_Countries")
africa_list <- read.csv("africa.csv",sep=";")

africa <-subset(ctries, ctries[[1]]  == as.character(africa_list[1,1]))
for ( i in 2:length(africa_list[,1])) {
  africa <- rbind(africa,(subset(ctries, ctries[[1]]  == as.character(africa_list[i,1])))) 
}

#Develop ggplot dataframes for continents
ctries@data$id = rownames(ctries@data)
ctries_points = fortify(ctries, region="id")                  
ctries_df = merge(ctries_points,ctries@data, by="id")

africa@data$id = rownames(africa@data)
africa_points = fortify(africa, region="id")                  
africa_df = merge(africa_points,africa@data, by="id")

#climate data
setwd("C:/Users/CR-C003/Dropbox/Worldclim")
bioclim <- stack(list.files(pattern="bio"))

#select climate variables for modelling
rs <- gridSample(vigna[,c("decimalLongitude","decimalLatitude")], bioclim[[1]], n=1)
spp_clim <- extract(bioclim,rs)
spp_clim <- na.omit(spp_clim)
setwd("C:/Users/CR-C003/Dropbox/Gap analysis/Vigna")
write.csv(as.data.frame(cor(spp_clim)),"cor.csv",row.names = F)

setwd("C:/Users/CR-C003/Dropbox/Worldclim")
bio_vigna <- stack("bio2.tif",
                   "bio5.tif",
                   "bio8.tif",
                   "bio11.tif", 
                   "bio12.tif", 
                   "bio14.tif", 
                   "bio17.tif",
                   "bio18.tif",
                   "bio19.tif")
#PARAMETERS####

#define diameter and resolution of circular neighboorhood in minutes
dmtr <- 120
res <- 30


#FUNCTIONS#####

#THRESHOLD SDM MODEL
umbral <- function(x){
  e <- evaluate(x, a=bg, xm, x=bio_vigna)
  umbral <- e@t[which.max(e@TPR + e@TNR)]
  m <- c(0, umbral, 0, umbral, 1 ,1)
  rclmat <- matrix(m, ncol=3, byrow=TRUE)
  return(rclmat)}

#DEVELOP MAXENT MODEL 
cl <- cmpfun(run_maxent)
run_maxent <- function(x) {
  xm <- maxent(bio_vigna, 
               x, a=pts, 
               removeDuplicates=TRUE, 
               path= paste("C:/Users/CR-C003/Dropbox/Gap analysis/Vigna/pot_dist/",spp[i],"/rep_",m, sep=""), argc=c("-r","nowarnings"))
  return(xm)}

#Develop record file cn
cn <- function(x,y,z){
  sp <- x[,c("decimalLongitude","decimalLatitude")]
  coord <- SpatialPointsDataFrame(sp, x, proj4string=CRS(as.character(NA)))
  
  # Final vector
  input.cn <- vector() 
  
  #cn
  for (i in 1:length(x[,1])){
    print(i)
    circle <- gBuffer(coord[i,], width=y/60/2)
    GT <- GridTopology(c((sp[i,1]-y/60/2+z/60/2),(sp[i,2]-y/60/2+z/60/2)),
                       cellsize = c(z/60,z/60),
                       cells.dim = c(y/z,(y/z)))
    GT.sp <- SpatialPoints(GT, proj4string = CRS(as.character(NA)))
    match <- over(GT.sp,circle)
    coords <- as.matrix(na.omit(cbind(over(GT.sp,circle),as.matrix(GT.sp@coords))))
    input.cn <- rbind(input.cn, merge(coords, x[i,]))
  }
  return(input.cn)}

#x = dataframe with coords
#y = diameter in minutes of cn
#z = resolution of future cells

#Develop dataframe with percentiles of climate gap
clim_gap <- function(x){
  perc_bio1 <- matrix(NA, ncol=3,nrow =(length(spp)))
  for (i in 1:length(spp)) {
    if (length(subset(vigna_clim_gb, GRIN.taxa == spp[i])[,x]) < 2)
    {perc_bio1[i,1] <- spp[i]
    perc_bio1[i,2] <- "NA"
    perc_bio1[i,3] <- "NA"} else {perc_bio1[i,1] <- spp[i]
    ptle <- quantile(subset(vigna_clim_gb, GRIN.taxa == spp[i])[,x], c(0, 1), na.rm=T) 
    ifelse(length(subset(vigna_clim_hb, GRIN.taxa == spp[i])[,x]) ==0, perc_bio1[i,2] <- 0,
           perc_bio1[i,2] <- ecdf(subset(vigna_clim_hb, GRIN.taxa == spp[i])[,x])(ptle[1])*100)
    ifelse(length(subset(vigna_clim_hb, GRIN.taxa == spp[i])[,x]) ==0, perc_bio1[i,3] <- 100,
           perc_bio1[i,3] <- ecdf(subset(vigna_clim_hb, GRIN.taxa == spp[i])[,x])(ptle[2])*100)
    }
  }
  perc_bio1 <- as.data.frame(perc_bio1)
  colnames(perc_bio1) <- c("taxa","lower_limit", "higher_limit")
  perc_bio1$lower_limit <- as.numeric(as.character(perc_bio1$lower_limit)) +1
  perc_bio1$higher_limit <- 101 - as.numeric(as.character(perc_bio1$higher_limit)) 
  perc_bio1 <- na.omit(perc_bio1)
  return(perc_bio1)}

#SUBSAMPLING TRAINING DATA
train_points <- function(x) {
  #calculate largest distance
  largedist <- max(pointDistance(x,longlat=FALSE), na.rm=TRUE)
  #develop convex hul
  hull <- convHull(x,lonlat=TRUE)
  #extent convex hull
  ext.hull <- gBuffer(hull@polygons, width=0.1*largedist)
  #define raster
  r <- raster(ext.hull)
  # set the resolution of the cells to (for example) 1 degree
  res(r) <- 0.025*largedist
  #train 
  train <- gridSample(x, r, n=1)
  return(train)
}

#MAXENT MODELLING########
bio_vigna <- crop(bio_vigna,
                  c(xmn = -180,
                    xmx = 180,
                    ymn = min(vigna[,c("decimalLatitude")])-dmtr/60/2 - res/60,
                    ymx = max(vigna[,c("decimalLatitude")])+dmtr/60/2 + res/60))


#all species included in modelling exercise
spp <- unique(vigna[, "GRIN.taxa"])

#Develop folder 
for (i in 1:length(spp)){dir.create(path= paste("C:/Users/CR-C003/Dropbox/Gap analysis/Vigna/pot_dist/",spp[i],sep=""))}


for (i in 5:length(spp)){
  print(as.character(spp)[[i]])  
  
  gc()  
  #check if sufficient training points
  sP <- subset(vigna, GRIN.taxa == spp[i])[,c("decimalLongitude","decimalLatitude")]
  rand <- sample(c(1:length(sP[,1])),length(sP[,1]), replace = F)
  train <- cbind(sP,rand)
  train <- train[,1:2]
  if (length(train[,1]) < 3) {print("NA")} else
  {if (length(train_points(train)[,1]) < 3) {print("NA")} else {
    
    sP <- subset(vigna, GRIN.taxa == spp[i])[,c("decimalLongitude","decimalLatitude")]
    
    #//TRAINING DATA\\
    training <- list()
    
    for (l in 1:3){
      
      #Training points
      rand <- sample(c(1:length(sP[,1])),length(sP[,1]), replace = F)
      train <- cbind(sP,rand)
      train <- train[,1:2]
      training[[l]] <- train_points(train)
      
    }
    
    #mask map with species polygon extention
    largedist <- max(pointDistance(sP,longlat=FALSE), na.rm=TRUE)   
    hull <- convHull(sP,lonlat=TRUE)
    ext.hull <- gBuffer(hull@polygons, width=0.1*largedist) 
    ifelse(extent(ext.hull)[1] < -180, ext.hull@bbox[1] <- -180, extent(ext.hull)[1])
    ifelse(extent(ext.hull)[2] > 180, ext.hull@bbox[3] <- 180,extent(ext.hull)[2])
    ifelse(extent(ext.hull)[3] < -90, ext.hull@bbox[2] <- -90, extent(ext.hull)[3])
    ifelse(extent(ext.hull)[4] > 90, ext.hull@bbox[4] <- 90, extent(ext.hull)[4])
    ext.hull@proj4string <- CRS("+proj=longlat +datum=WGS84")
    
    #background points
    bg <- spsample(ext.hull,10*length(train[,1]), type="random")
    s <- extract(bio_vigna[[1]],bg)
    pts <- as.data.frame(na.omit(cbind(bg@coords,s))[,1:2])
    
    #train model over training sets  
    xm_list <- list()
    px_clim <- list() 
    reclass <- list()
    for (m in 1:length(training)){
      
      #develop model
      
      #run maxent with training data subsets (training[[m]] == x)
      xm <- run_maxent(training[[m]])
      
      
      xm_list[[m]] <- xm
      
      #model predict
      px_clim[[m]] <- predict(xm, bio_vigna, progress='text')
      reclass[[m]] <- umbral(training[[m]])
      
    }
    
    #average model result
    px_clim_mean <- mean(stack(px_clim))
    
    #cut threshold current
    
    # determine average threshold presence absence
    reclass_mean <- matrix(0,nrow =2, ncol =3)
    for (mm in 1:length(training)){reclass_mean <- reclass_mean +(reclass[[mm]]/length(training))}
    
    #develop presence absence map
    bioclim_threshold <- reclassify(px_clim_mean, reclass_mean)
    
    #cut gradient map with absence map
    bioclim_cut <- overlay(px_clim_mean, bioclim_threshold, fun=function(px_clim_mean,bioclim_threshold) px_clim_mean*bioclim_threshold)
    
    #cut according to threshold
    bioclim_cut <- mask(bioclim_cut, ext.hull)
    bioclim_threshold <- mask(bioclim_threshold, ext.hull)
    
    #Convert 0 values in NA
    bioclim_threshold[bioclim_threshold == 0] <- NA
    
    #write raster
    setwd(paste("C:/Users/CR-C003/Dropbox/Gap analysis/Vigna/pot_dist/",spp[i], sep="")) 
    writeRaster(bioclim_threshold,file=paste(spp[i],"_bioclim_threshold.asc",sep=""),overwrite=TRUE)
    
  }}
}
