library(raster)
library(rgdal)
library(maptools)
library(rgeos)
library(sp)
library(ggplot2)
library(ggrepel)
library(dismo)
library(compiler)

#Set WD and memory options####
options( java.parameters = "-Xmx6g")

output <- "C:/Users/CR-C003/Dropbox"
output <- "C:/Users/maarten.vanzonneveld/Dropbox" 

#DATA INPUT####
#Enter Vigna records
setwd(paste(output,"/Gap analysis/Vigna",sep=""))
vigna <- read.csv("Vigna_clean_5.csv",sep=",")

#Ensure that coordinates are numeric variables and remove NAs
vigna$decimalLongitude <- as.numeric(as.character(vigna$decimalLongitude))
vigna <- vigna[!is.na(vigna$decimalLongitude),]

vigna$decimalLatitude <- as.numeric(as.character(vigna$decimalLatitude))
vigna <- vigna[!is.na(vigna$decimalLongitude),]

#REMOVE SPP NOT INCLUDED IN THE GENEPOOL
#species not included in the analysis
spp_na <- c("V. candida", "V. gentryi", "v. juncea", "V. triphylla", 
            "V. kokii", "V. lanceolata", "V. nigritia", 
            "V. schimperi","V. spp.", "V. venusta")

for (i in 1:length(spp_na)){vigna <- subset(vigna,!GRIN.taxa == as.character(spp_na[i]))}

length(unique(vigna[,"GRIN.taxa"]))
#Enter country data
setwd(paste(output,"/World",sep=""))
ctries <- readOGR("World_Countries.shp", "World_Countries")

#African countries 
africa_list <- read.csv("africa.csv",sep=";")
africa <-subset(ctries, ctries[[1]]  == as.character(africa_list[1,1]))
for ( i in 2:length(africa_list[,1])) {
africa <- rbind(africa,(subset(ctries, ctries[[1]]  == as.character(africa_list[i,1])))) 
}

#Develop ggplot dataframe for continent
ctries@data$id = rownames(ctries@data)
ctries_points = fortify(ctries, region="id")                  
ctries_df = merge(ctries_points,ctries@data, by="id")

#Develop ggplot dataframe for Africa
africa@data$id = rownames(africa@data)
africa_points = fortify(africa, region="id")                  
africa_df = merge(africa_points,africa@data, by="id")

#PARAMETERS####

#define diameter and resolution of circular neighboorhood in minutes
dmtr <- 120
res <- 30

#FUNCTIONS#####

#Develop record file cn
cn <- function(x,y,z){
sp <- x[,c("decimalLongitude","decimalLatitude")]
coord <- SpatialPointsDataFrame(sp, x, proj4string=CRS(as.character(NA)))

# Final vector
input.cn <- vector() 

#cn
for (i in 1:length(x[,1])){
print(i)
circle <- gBuffer(coord[i,], width=y/60/2)
GT <- GridTopology(c((sp[i,1]-y/60/2+z/60/2),(sp[i,2]-y/60/2+z/60/2)),
                   cellsize = c(z/60,z/60),
                   cells.dim = c(y/z,(y/z)))
GT.sp <- SpatialPoints(GT, proj4string = CRS(as.character(NA)))
match <- over(GT.sp,circle)
coords <- as.matrix(na.omit(cbind(over(GT.sp,circle),as.matrix(GT.sp@coords))))
input.cn <- rbind(input.cn, merge(coords, x[i,]))
}
return(input.cn)}

#x = dataframe with coords
#y = diameter in minutes of cn
#z = resolution of future cells


#MAKE RASTER####
#raster with extent for richness maps
setwd(paste(output,"/Gap analysis/Vigna/pot_dist/V. mungo",sep=""))
basis <- raster("V. mungo_bioclim_threshold.asc")
r <- basis
res(r) <- res/60
values(r) <- c(1:ncell(r))

#CIRCULAR NEIGHBOORHOOD####
vigna_cn <- cn(vigna,dmtr,res)

vigna_hb_cn <- rbind(subset(vigna_cn, SOURCE == "GBIF NEW"),
                     subset(vigna_cn, SOURCE == "GBIF GEO"),
                     subset(vigna_cn, SOURCE == "TOMOOKA"),
                     subset(vigna_cn, SOURCE == "GBIF"),
                     subset(vigna_cn, SOURCE == "GBIF EXTRA"))

vigna_gb_cn <- rbind(subset(vigna_cn, SOURCE == "WIEWS"),
                     subset(vigna_cn, SOURCE == "WORLDVEG"),
                     subset(vigna_cn, SOURCE == "GBIF LIVE"))

#Raster template to carry out point to grids 
v <- r
values(v)  <- 0

#Point to grid
div_hb_map <- rasterize(vigna_hb_cn[,2:3],
                        v,
                        background = 0,
                        (vigna_hb_cn[,"GRIN.taxa"]), 
                        fun = function(x,...){length(unique(na.omit(x)))})

div_gb_map <- rasterize(vigna_gb_cn[,2:3],
                        v,
                        background = 0,
                        (vigna_gb_cn[,"GRIN.taxa"]), 
                        fun = function(x,...){length(unique(na.omit(x)))})

div_map <- rasterize(vigna_cn[,2:3],
                     v,
                     (vigna_cn[,"GRIN.taxa"]), 
                     fun = function(x,...){length(unique(na.omit(x)))})

#sampled richenss map for overlay with modelled richness
div_map1 <- div_map
div_map1[is.na(div_map1[])] <- 0 
values(div_map1) 

#MODELLED RICHNESS
r <- basis
values(r) <- c(1:ncell(r))
v <- r
values(v)  <- 0

setwd(paste(output,"/Gap analysis/Vigna/pot_dist",sep=""))
list_spp <- list.files(pattern="V.") 

div_maps <- list()
for (i in 1:length(list_spp)){
setwd(paste(output,"/Gap analysis/Vigna/pot_dist/",list_spp[i],sep=""))
if (length(list.files(pattern="V.")) == 1) 
{spp_r <- raster(paste(list_spp[i],"_bioclim_threshold.asc",sep=""))
div_maps[[i]] <- spp_r} else
{spp_map <- subset(vigna_cn, GRIN.taxa == list_spp[i])
div_maps[[i]] <- rasterize(spp_map[,2:3],
                     v,
                     (spp_map[,"GRIN.taxa"]), 
                     fun = function(x,...){length(unique(na.omit(x)))})}  
}  

div_maps_agg <- list()
for (i in 1:length(div_maps)) {
  print(i)
  div_maps_agg[[i]] <- aggregate(div_maps[[i]], 12, max)}
mod_rich <- sum(stack(div_maps_agg),na.rm=T)
mod_rich <- crop(mod_rich, div_map)
plot(mod_rich)

#OVERLAY####
#sampled richness
div_over <- mask(div_map,ctries)
div_asia  <- crop(div_over,c(50,175,-40,55))
div_africa <- mask(div_over,africa)


#gap analysis sampled richness
gap_over <- overlay(div_gb_map, div_hb_map, fun=function(x,y){return(x-y)})
gap_over <- mask(gap_over,div_map)
gap_over <- mask(gap_over,ctries)
overlay_asia  <- crop(gap_over,c(50,175,-40,55))
overlay_africa <- mask(gap_over,africa)

#Gap analysis of modeled richness 
mod_over <- overlay(div_map1, mod_rich, fun=function(x,y){return(x-y)})
mod_over <- mask(mod_over,ctries)
mod_over_asia  <- crop(mod_over,c(50,175,-40,55))
mod_over_africa <- mask(mod_over,africa)

setwd(paste(output,"/Gap analysis/Vigna",sep=""))

#GGPLOT MAPS####

#Sampled richness of Africa
k <- as.data.frame(div_africa, xy =TRUE)
k <- na.omit(k)

Filename= "vigna richness africa.tiff"
map2 <- ggplot(k,aes(x,y)) + 
  geom_polygon(data = africa_df,
               size=0.25, colour = "darkgray", fill = "lightgray",
               aes(x=long,y=lat, group = group)) + 
  geom_raster(aes(fill= k[,3])) +
  scale_fill_gradient(low = "light yellow", high= "blue",na.value= "lightgray",  name="No. taxa") +
  geom_path(data = africa_df, size=0.1, colour = "grey50", aes(x=long,y=lat, group = group), size = 0.25)+ 
  coord_quickmap(xlim = range (-25,55), ylim = range (-35,35)) +
  theme(
    panel.grid.minor = element_blank(), 
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill="transparent",colour=NA),
    plot.background = element_rect(fill="transparent",colour=NA),
    panel.border = element_rect(colour = "black", fill=NA),
    axis.text=element_text(size=18),
    axis.title=element_text(size=26),
    plot.title=element_text(size=28,face="bold", vjust=1, hjust=0.5))+
  xlab("Longitude") + 
  ylab("Latitude") +
  ggtitle("Sampled richness") +
  theme(legend.position = c(0.22,0.3),
        plot.title = element_text(hjust=0.5),
        legend.text = element_text(size = 12),
        legend.key.height = unit(0.5, "cm"),
        legend.key.width = unit(0.5, "cm"),
        legend.title=element_text(size=14))
ggsave(filename = Filename, map2, dpi=300)
plot(map2)


#Sampled richness SE Asia
k <- as.data.frame(div_asia, xy =TRUE)
k <- na.omit(k)

Filename= "vigna richness asia.tiff"
map2 <- ggplot(k,aes(x,y)) + 
  geom_polygon(data = ctries_df,
               size=0.25, colour = "darkgray", fill = "lightgray",
               aes(x=long,y=lat, group = group)) + 
  geom_raster(aes(fill= k[,3])) +
  scale_fill_gradient(low = "light yellow", high= "blue",na.value= "lightgray",  name="No. taxa") +
  geom_path(data = ctries_df, size=0.1, colour = "grey50", aes(x=long,y=lat, group = group), size = 0.25)+ 
  coord_quickmap(xlim = range (65,155), ylim = range (-30,45)) +
  theme(
    panel.grid.minor = element_blank(), 
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill="transparent",colour=NA),
    plot.background = element_rect(fill="transparent",colour=NA),
    panel.border = element_rect(colour = "black", fill=NA),
    axis.text=element_text(size=18),
    axis.title=element_text(size=26),
    plot.title=element_text(size=28,face="bold", vjust=1, hjust=0.5, color = "white"))+
  xlab("Longitude") + 
  ylab("Latitude") +
  ggtitle("Sampled richness") +
  theme(legend.position = c(0.18,0.25),
        plot.title = element_text(hjust=0.5),
        legend.text = element_text(size = 12),
        legend.key.height = unit(0.5, "cm"),
        legend.key.width = unit(0.5, "cm"),
        legend.title=element_text(size=14))
ggsave(filename = Filename, map2, dpi=300)


#Gap analysis of sampled richness Africa
k <- as.data.frame(overlay_africa, xy =TRUE)
k <- na.omit(k)

Filename= "vigna sampled gaps africa.tiff"
map2 <- ggplot(k,aes(x,y)) + 
  geom_polygon(data = africa_df,
               size=0.25, colour = "darkgray", fill = "lightgray",
               aes(x=long,y=lat, group = group)) + 
  geom_raster(aes(fill= k[,3])) +
  scale_fill_gradient2(high = "darkgreen", mid= "light yellow", low= "red",na.value= "lightgray", midpoint = 0, name="No. taxa") +
  geom_path(data = africa_df, size=0.1, colour = "grey50", aes(x=long,y=lat, group = group), size = 0.25)+ 
  coord_quickmap(xlim = range (-25,55), ylim = range (-35,35)) +
  theme(
    panel.grid.minor = element_blank(), 
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill="transparent",colour=NA),
    plot.background = element_rect(fill="transparent",colour=NA),
    panel.border = element_rect(colour = "black", fill=NA),
    axis.text=element_text(size=18),
    axis.title=element_text(size=26),
    plot.title=element_text(size=28,face="bold", vjust=1, hjust=0.5))+
  xlab("Longitude") + 
  ylab("Latitude") +
  ggtitle("Sampled collection gaps") +
  theme(legend.position = c(0.22,0.3),
        plot.title = element_text(hjust=0.5),
        legend.text = element_text(size = 12),
        legend.key.height = unit(0.5, "cm"),
        legend.key.width = unit(0.5, "cm"),
        legend.title=element_text(size=14))
ggsave(filename = Filename, map2, dpi=300)
plot(map2)

#Gap analysis of sampled richness SE Asia
k <- as.data.frame(overlay_asia, xy =TRUE)
k <- na.omit(k)

Filename= "vigna sampled gaps asia.tiff"
map2 <- ggplot(k,aes(x,y)) + 
  geom_polygon(data = ctries_df,
               size=0.25, colour = "darkgray", fill = "lightgray",
               aes(x=long,y=lat, group = group)) + 
  geom_raster(aes(fill= k[,3])) +
  scale_fill_gradient2(high = "darkgreen", mid= "light yellow", low= "red",na.value= "lightgray", midpoint = 0, name="No. taxa") +
  geom_path(data = ctries_df, size=0.1, colour = "grey50", aes(x=long,y=lat, group = group), size = 0.25)+ 
  coord_quickmap(xlim = range (65,155), ylim = range (-30,45)) +
  theme(
    panel.grid.minor = element_blank(), 
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill="transparent",colour=NA),
    plot.background = element_rect(fill="transparent",colour=NA),
    panel.border = element_rect(colour = "black", fill=NA),
    axis.text=element_text(size=18),
    axis.title=element_text(size=26),
    plot.title=element_text(size=28,face="bold", vjust=1, hjust=0.5, color = "white"))+
  xlab("Longitude") + 
  ylab("Latitude") +
  ggtitle("Sampled collection gaps") +
  theme(legend.position = c(0.18,0.25),
        plot.title = element_text(hjust=0.5),
        legend.text = element_text(size = 12),
        legend.key.height = unit(0.5, "cm"),
        legend.key.width = unit(0.5, "cm"),
        legend.title=element_text(size=14))
ggsave(filename = Filename, map2, dpi=300)


#Gap analysis of modelled richness Africa
k <- as.data.frame(mod_over_africa, xy =TRUE)
k <- na.omit(k)

Filename= "vigna modelled gaps africa.tiff"
map2 <- ggplot(k,aes(x,y)) + 
  geom_polygon(data = africa_df,
               size=0.25, colour = "darkgray", fill = "lightgray",
               aes(x=long,y=lat, group = group)) + 
  geom_raster(aes(fill= k[,3])) +
  scale_fill_gradient2(high = "darkgreen", mid= "light yellow", low= "red",na.value= "lightgray", midpoint = 0, name="No. taxa") +
  geom_path(data = africa_df, size=0.1, colour = "grey50", aes(x=long,y=lat, group = group), size = 0.25)+ 
  coord_quickmap(xlim = range (-25,55), ylim = range (-35,35)) +
  theme(
    panel.grid.minor = element_blank(), 
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill="transparent",colour=NA),
    plot.background = element_rect(fill="transparent",colour=NA),
    panel.border = element_rect(colour = "black", fill=NA),
    axis.text=element_text(size=18),
    axis.title=element_text(size=26),
    plot.title=element_text(size=28,face="bold", vjust=1, hjust=0.5))+
  xlab("Longitude") + 
  ylab("Latitude") +
  ggtitle("Modelled collection gaps") +
  theme(legend.position = c(0.22,0.3),
        plot.title = element_text(hjust=0.5),
        legend.text = element_text(size = 12),
        legend.key.height = unit(0.5, "cm"),
        legend.key.width = unit(0.5, "cm"),
        legend.title=element_text(size=14))
ggsave(filename = Filename, map2, dpi=300)


#Gap analysis of modelled richness SE Asia
k <- as.data.frame(mod_over_asia, xy =TRUE)
k <- na.omit(k)

Filename= "vigna modelled gaps asia.tiff"
map2 <- ggplot(k,aes(x,y)) + 
  geom_polygon(data = ctries_df,
               size=0.25, colour = "darkgray", fill = "lightgray",
               aes(x=long,y=lat, group = group)) + 
  geom_raster(aes(fill= k[,3])) +
  scale_fill_gradient2(high = "darkgreen", mid= "light yellow", low= "red",na.value= "lightgray", midpoint = 0, name="No. taxa") +
  geom_path(data = ctries_df, size=0.1, colour = "grey50", aes(x=long,y=lat, group = group), size = 0.25)+ 
  coord_quickmap(xlim = range (65,155), ylim = range (-30,45)) +
  theme(
    panel.grid.minor = element_blank(), 
    panel.grid.major = element_blank(),
    panel.background = element_rect(fill="transparent",colour=NA),
    plot.background = element_rect(fill="transparent",colour=NA),
    panel.border = element_rect(colour = "black", fill=NA),
    axis.text=element_text(size=18),
    axis.title=element_text(size=26),
    plot.title=element_text(size=28,face="bold", vjust=1, hjust=0.5, color = "white"))+
  xlab("Longitude") + 
  ylab("Latitude") +
  ggtitle("Modelled collection gaps") +
  theme(legend.position = c(0.18,0.25),
        plot.title = element_text(hjust=0.5),
        legend.text = element_text(size = 12),
        legend.key.height = unit(0.5, "cm"),
        legend.key.width = unit(0.5, "cm"),
        legend.title=element_text(size=14))
ggsave(filename = Filename, map2, dpi=300)

